@@ -1,15 +1,17 @@
 package App::idxdb;
 
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2021-01-19'; # DATE
+our $DATE = '2021-01-21'; # DATE
 our $DIST = 'App-idxdb'; # DIST
-our $VERSION = '0.002'; # VERSION
+our $VERSION = '0.003'; # VERSION
 
 use 5.010001;
 use strict;
 use warnings;
 use Log::ger;
 
+use Data::Clone qw(clone);
+use DateTime;
 use File::chdir;
 
 sub _set_args_default {
@@ -50,6 +52,8 @@ sub _init {
     $App::idxdb::state;
 }
 
+my $today = DateTime->today;
+
 our %SPEC;
 
 $SPEC{':package'} = {
@@ -82,6 +86,40 @@ my %ownership_fields = (
 );
 my @ownership_fields = sort keys %ownership_fields;
 
+my %daily_trading_summary_fields = (
+    'Bid' => {type=>'price'},
+    'BidVolume' => {type=>'volume'},
+    'Change' => {type=>'price'},
+    'Close' => {type=>'price'},
+    'DelistingDate' => {type=>'date'},
+    'FirstTrade' => {type=>'price'},
+    'ForeignBuy' => {type=>'volume'},
+    'ForeignSell' => {type=>'volume'},
+    'ForeignNetBuy' => {type=>'volume'}, # calculated
+    'AccumForeignBuy'    => {type=>'accum_volume'}, # calculated
+    'AccumForeignSell'   => {type=>'accum_volume'}, # calculated
+    'AccumForeignNetBuy' => {type=>'accum_volume'}, # calculated
+    'Frequency' => {type=>'freq'},
+    'High' => {type=>'price'},
+    'IDStockSummary' => {type=>'str'},
+    'IndexIndividual' => {type=>'index'},
+    'ListedShares' => {type=>'num'},
+    'Low' => {type=>'price'},
+    'NonRegularFrequency' => {type=>'freq'},
+    'NonRegularValue' => {type=>'money'},
+    'NonRegularVolume' => {type=>'volume'},
+    'Offer' => {type=>'price'},
+    'OfferVolume' => {type=>'volume'},
+    'OpenPrice' => {type=>'price'},
+    'Previous' => {type=>'price'},
+    'Remarks' => {type=>'str'},
+    'TradebleShares' => {type=>'num'},
+    'Value' => {type=>'money'},
+    'Volume' => {type=>'volume'},
+    'WeightForIndex' => {type=>'num'},
+);
+my @daily_trading_summary_fields = sort keys %daily_trading_summary_fields;
+
 our %args_common = (
     dbpath => {
         summary => 'Path for SQLite database',
@@ -116,16 +154,30 @@ our %arg0_stocks = (
 
 our %argsopt_filter_date = (
     date_start => {
-        schema => ['date*', 'x.perl.coerce_to' => 'DateTime'],
+        schema => ['date*', 'x.perl.coerce_to' => 'DateTime', 'x.perl.coerce_rules'=>['From_str::natural']],
         tags => ['category:filtering'],
+        default => $today->clone->subtract(days=>30)->epoch,
+        cmdline_aliases => {
+            week     => {is_flag=>1, code=>sub {$_[0]{date_start} = $today->clone->subtract(days=>  7); $_[0]{date_end} = $today}},
+            month    => {is_flag=>1, code=>sub {$_[0]{date_start} = $today->clone->subtract(days=> 30); $_[0]{date_end} = $today}},
+            '3month' => {is_flag=>1, code=>sub {$_[0]{date_start} = $today->clone->subtract(days=> 90); $_[0]{date_end} = $today}},
+            '6month' => {is_flag=>1, code=>sub {$_[0]{date_start} = $today->clone->subtract(days=>180); $_[0]{date_end} = $today}},
+            # YTD
+            'year'   => {is_flag=>1, code=>sub {$_[0]{date_start} = $today->clone->subtract(days=>   365); $_[0]{date_end} = $today}},
+            '3year'  => {is_flag=>1, code=>sub {$_[0]{date_start} = $today->clone->subtract(days=> 3*365); $_[0]{date_end} = $today}},
+            '5year'  => {is_flag=>1, code=>sub {$_[0]{date_start} = $today->clone->subtract(days=> 5*365); $_[0]{date_end} = $today}},
+            '10year' => {is_flag=>1, code=>sub {$_[0]{date_start} = $today->clone->subtract(days=>10*365); $_[0]{date_end} = $today}},
+        },
     },
     date_end => {
-        schema => ['date*', 'x.perl.coerce_to' => 'DateTime'],
+        schema => ['date*', 'x.perl.coerce_to' => 'DateTime', 'x.perl.coerce_rules'=>['From_str::natural']],
         tags => ['category:filtering'],
+        default => $today->epoch,
     },
 );
 
-my $sch_ownership_field = ['str*'=>{in=>\@ownership_fields, 'x.in.summaries'=>[map {$ownership_fields{$_}} @ownership_fields]}];
+my $sch_ownership_field             = ['str*'=>{in=>\@ownership_fields, 'x.in.summaries'=>[map {$ownership_fields{$_}} @ownership_fields]}];
+my $sch_daily_trading_summary_field = ['str*'=>{in=>\@daily_trading_summary_fields}];
 
 our %argopt_field_ownership = (
     field => {
@@ -139,7 +191,7 @@ our %argopt_fields_ownership = (
     fields => {
         'x.name.is_plural' => 1,
         'x.name.singular' => 'field',
-        schema => ['array*', of=>$sch_ownership_field],
+        schema => ['array*', of=>$sch_ownership_field, 'x.perl.coerce_rules'=>['From_str::comma_sep']],
         tags => ['category:field_selection'],
         default => ['LocalTotal', 'ForeignTotal'],
         cmdline_aliases => {
@@ -150,6 +202,32 @@ our %argopt_fields_ownership = (
     },
 );
 
+our %argopt_field_daily_trading_summary = (
+    field => {
+        schema => $sch_daily_trading_summary_field,
+        tags => ['category:field_selection'],
+        default => 'AccumForeignNetBuy',
+    },
+);
+
+our %argopt_fields_daily_trading_summary = (
+    fields => {
+        'x.name.is_plural' => 1,
+        'x.name.singular' => 'field',
+        schema => ['array*', of=>$sch_daily_trading_summary_field, 'x.perl.coerce_rules'=>['From_str::comma_sep']],
+        tags => ['category:field_selection'],
+        default => ['Volume','Value','ForeignNetBuy'],
+        cmdline_aliases => {
+            fields_all => {is_flag=>1, code=>sub { $_[0]{fields} = \@daily_trading_summary_fields }},
+            #fields_foreign => {is_flag=>1, code=>sub { $_[0]{fields} = [grep {/Foreign/} @ownership_fields] }},
+            #fields_local   => {is_flag=>1, code=>sub { $_[0]{fields} = [grep {/Local/} @ownership_fields] }},
+        },
+    },
+);
+
+our %argopt_fields_daily_trading_summary_for_graph = %{ clone(\%argopt_fields_daily_trading_summary) };
+$argopt_fields_daily_trading_summary_for_graph{fields}{default} = ['AccumForeignNetBuy'];
+
 $SPEC{update} = {
     v => 1.1,
     summary => 'Update data',
@@ -185,6 +263,7 @@ sub update {
     {
         my $table_exists = DBIx::Util::Schema::table_exists($dbh, 'meta');
         last if $table_exists;
+        log_info "Creating meta table ...";
         $dbh->do("CREATE TABLE meta (name TEXT PRIMARY KEY, value TEXT)");
     }
 
@@ -222,7 +301,7 @@ sub update {
 
   UPDATE_DAILY_TRADING_SUMMARY:
     {
-        log_info "Updating daily trading summary ...";
+        log_trace "Updating daily trading summary ...";
         my $table_exists = DBIx::Util::Schema::table_exists($dbh, 'daily_trading_summary');
         my @table_fields;
         if ($table_exists) {
@@ -238,7 +317,7 @@ sub update {
                 log_trace "Processing file $CWD/$filename ...";
                 my $date = "$1-$2-${3}";
                 if ($table_exists && $dbh->selectrow_array(q(SELECT 1 FROM daily_trading_summary WHERE "Date" = ?), {}, $date)) {
-                    log_debug "Data for date $date already exist, skipping this date";
+                    log_trace "Data for date $date already exist, skipping this date";
                     next FILENAME;
                 }
                 open my $fh, "gzip -cd $filename |" or die "Can't open $filename: $!";
@@ -262,7 +341,7 @@ sub update {
                     $dbh->do("CREATE UNIQUE INDEX ix_daily_trading_summary__Date__StockCode ON daily_trading_summary(Date,StockCode)");
                     $table_exists++;
                 }
-
+                log_info "Inserting daily trading summary for $date ..." if @$data;
                 my $sql = "INSERT INTO daily_trading_summary (".join(",", map {qq("$_")} @table_fields).") VALUES (".join(",", map {"?"} @table_fields).")";
                 #log_warn $sql;
                 my $sth_ins_daily_trading_summary = $dbh->prepare($sql);
@@ -280,7 +359,7 @@ sub update {
 
   UPDATE_OWNERSHIP:
     {
-        log_info "Updating stock ownership ...";
+        log_trace "Updating stock ownership ...";
         my $table_exists = DBIx::Util::Schema::table_exists($dbh, 'stock_ownership');
         my @table_fields;
         if ($table_exists) {
@@ -324,6 +403,7 @@ sub update {
                 }
 
                 unless ($table_exists) {
+                    log_info "Creating table 'stock_ownership' ...";
                     my @table_field_defs;
                     push @table_fields    , "date";
                     push @table_field_defs, "date TEXT NOT NULL";
@@ -345,6 +425,7 @@ sub update {
                     $table_exists++;
                 }
 
+                log_info "Inserting stock ownership data for $date ...";
                 my $sql = "INSERT INTO stock_ownership (".join(",", map {qq("$_")} @table_fields).") VALUES (".join(",", map {"?"} @table_fields).")";
                 #log_warn $sql;
                 my $sth_ins_stock_ownership = $dbh->prepare($sql);
@@ -475,7 +556,7 @@ sub graph_ownership {
             xlabel   => 'date',
             ylabel   => "\%$field",
             timeaxis => 'x',
-            xtics    => {labelfmt=>'%Y-%m-%d'},
+            xtics    => {labelfmt=>'%Y-%m-%d', rotate=>"30 right"},
             #yrange   => [0, 100],
         );
         for my $stock (@$stocks) {
@@ -512,6 +593,7 @@ $SPEC{graph_ownership_composition} = {
 sub graph_ownership_composition {
     require Chart::Gnuplot;
     require Color::RGB::Util;
+    require ColorTheme::Distinct::WhiteBG;
     require File::Temp;
 
     my %args = @_;
@@ -566,41 +648,14 @@ sub graph_ownership_composition {
             xlabel   => 'date',
             ylabel   => "\%",
             timeaxis => 'x',
-            xtics    => {labelfmt=>'%Y-%m-%d'},
+            xtics    => {labelfmt=>'%Y-%m-%d', rotate=>"30 right",
+                     },
             #yrange   => [0, 100],
         );
         my $i = -1;
 
-        # ugh, the colors are horrible
-        #srand(2);
-        #my @colors = Color::RGB::Util::rand_rgb_colors({light_color=>undef, hash_prefix=>1}, @fields+0);
-
-        # using http://phrogz.net/css/distinct-colors.html. ugh, not much
-        # better.
-        #my @colors = (
-        #    '#ff0000',
-        #    '#e50000',
-        #    '#590000',
-        #    '#8c6969',
-        #    '#332626',
-        #    '#994526',
-        #    '#f2c6b6',
-        #    '#ff8c40',
-        #    '#4c2900',
-        #    '#b28959',
-        #);
-
-        #  manual is best
-        my @colors = qw(
-                           blue
-                           green
-                           red
-                           orange
-                           black
-                           magenta
-                           yellow
-                           gray
-                   );
+        my $theme = ColorTheme::Distinct::WhiteBG->new;
+        my @colors = map { '#'.$theme->get_item_color($_) } ($theme->list_items);
 
         for my $field (@fields) {
             $i++;
@@ -627,11 +682,165 @@ $SPEC{legend_ownership} = {
     summary => 'Show ownership legend (e.g. ForeignIB = foreign bank)',
     args => {
     },
+    examples => [
+        {
+            args=>{},
+            test=>0,
+        },
+    ],
 };
 sub legend_ownership {
     [200, "OK", \%ownership_fields];
 }
 
+$SPEC{table_daily_trading_summary} = {
+    v => 1.1,
+    summary => 'Show daily trading summary of some stock',
+    args => {
+        %arg0_stock,
+        %argsopt_filter_date,
+        %argopt_fields_daily_trading_summary,
+        total => {
+            schema => 'bool*',
+        },
+    },
+};
+sub table_daily_trading_summary {
+    my %args = @_;
+    my $stock = $args{stock};
+    my $fields = $args{fields};
+
+    my $state = _init(\%args, 'ro');
+    my $dbh = $state->{dbh};
+
+    my @wheres;
+    my @binds;
+    push @wheres, "StockCode=?";
+    push @binds, $stock;
+    if ($args{date_start}) {
+        push @wheres, "date >= '".$args{date_start}->ymd."'";
+    }
+    if ($args{date_end}) {
+        push @wheres, "date <= '".$args{date_end}->ymd."'";
+    }
+
+    my $sth = $dbh->prepare("SELECT * FROM daily_trading_summary WHERE ".join(" AND ", @wheres)." ORDER BY date");
+    $sth->execute(@binds);
+    my @rows;
+
+    my %total;
+    while (my $row = $sth->fetchrow_hashref) {
+        # calculated fields
+        #use DD; dd \@rows;
+        $row->{ForeignNetBuy}      = $row->{ForeignBuy} - $row->{ForeignSell};
+        $row->{AccumForeignBuy}    = (@rows ? $rows[-1]{AccumForeignBuy}    : 0) + $row->{ForeignBuy}     if grep {$_ eq 'AccumForeignBuy'} @$fields;
+        $row->{AccumForeignSell}   = (@rows ? $rows[-1]{AccumForeignSell}   : 0) + $row->{ForeignSell}    if grep {$_ eq 'AccumForeignSell'} @$fields;
+        $row->{AccumForeignNetBuy} = (@rows ? $rows[-1]{AccumForeignNetBuy} : 0) + $row->{ForeignNetBuy}  if grep {$_ eq 'AccumForeignNetBuy'} @$fields;
+
+        # calculate total
+        if ($args{total}) {
+            for my $f (@daily_trading_summary_fields) {
+                my $spec = $daily_trading_summary_fields{$f};
+                next unless $spec->{type} =~ /^(volume|money|freq)$/;
+                $total{$f} += $row->{$f}  if defined $row->{$f};
+            }
+        }
+
+        delete $row->{StockCode};
+        delete $row->{persen};
+        delete $row->{percentage};
+        delete $row->{ctime};
+        delete $row->{mtime};
+        for my $f (@daily_trading_summary_fields) { delete $row->{$f} unless (grep {$_ eq $f} @$fields) }
+        push @rows, $row;
+    }
+
+    if ($args{total} && @rows) {
+        for my $f (keys %total) {
+            delete $total{$f} unless (grep {$_ eq $f} @$fields);
+        }
+        $total{Date} = 'TOTAL';
+        push @rows, \%total;
+    }
+
+    my @ff  = ('Date', @$fields);
+    my (@ffa, @fffmt);
+    for (@ff) {
+        push @ffa  , (($daily_trading_summary_fields{$_}{type}//'') =~ /^(price|volume|accum_volume|money|freq|num)$/ ? 'right'  : undef);
+        push @fffmt, (($daily_trading_summary_fields{$_}{type}//'') =~ /^(price|volume|accum_volume|money|freq|num)$/ ? 'number' : undef);
+    }
+
+    [200, "OK", \@rows, {
+        'table.fields'       =>\@ff,
+        'table.field_aligns' =>\@ffa,
+        'table.field_formats'=>\@fffmt,
+    }];
+}
+
+$SPEC{graph_daily_trading_summary} = {
+    v => 1.1,
+    summary => 'Show graph from daily trading summary of some stock',
+    args => {
+        %arg0_stock,
+        %argsopt_filter_date,
+        %argopt_fields_daily_trading_summary_for_graph,
+    },
+};
+sub graph_daily_trading_summary {
+    require Chart::Gnuplot;
+    require Color::RGB::Util;
+    require ColorTheme::Distinct::WhiteBG;
+    require File::Temp;
+
+    my %args = @_;
+
+    my $res = table_daily_trading_summary(%args);
+    return $res unless $res->[0] == 200;
+    return [412, "No data"] unless @{ $res->[2] };
+
+    my $mindate = $res->[2][ 0]{Date};
+    my $maxdate = $res->[2][-1]{Date};
+    my @dates = map { $_->{Date} } @{ $res->[2] };
+
+    my ($tempfh, $tempfilename) = File::Temp::tempfile();
+    $tempfilename .= ".png";
+  DRAW_CHART: {
+        my @datasets;
+
+        my $chart = Chart::Gnuplot->new(
+            output   => $tempfilename,
+            title    => join(",", @{ $args{fields} })." of $args{stock} from $mindate to $maxdate",
+            xlabel   => 'date',
+            ylabel   => "",
+            timeaxis => 'x',
+            xtics    => {labelfmt=>'%Y-%m-%d', rotate=>"30 right",
+                     },
+        );
+        my $i = -1;
+
+        my $theme = ColorTheme::Distinct::WhiteBG->new;
+        my @colors = map { '#'.$theme->get_item_color($_) } ($theme->list_items);
+
+        for my $field (@{ $args{fields} }) {
+            $i++;
+            push @datasets, Chart::Gnuplot::DataSet->new(
+                xdata   => \@dates,
+                ydata   => [map { $_->{$field} } @{ $res->[2] }],
+                timefmt => '%Y-%m-%d',
+                title   => $field,
+                color   => $colors[$i],
+                style   => 'lines',
+            );
+        }
+        $chart->plot2d(@datasets);
+    }
+
+    require Browser::Open;
+    Browser::Open::open_browser("file:$tempfilename");
+
+    [200];
+}
+
 1;
 # ABSTRACT: Import data for stocks on the IDX (Indonesian Stock Exchange) and perform queries on them
 
@@ -647,7 +856,7 @@ App::idxdb - Import data for stocks on the IDX (Indonesian Stock Exchange) and p
 
 =head1 VERSION
 
-This document describes version 0.002 of App::idxdb (from Perl distribution App-idxdb), released on 2021-01-19.
+This document describes version 0.003 of App::idxdb (from Perl distribution App-idxdb), released on 2021-01-21.
 
 =head1 SYNOPSIS
 
@@ -658,6 +867,44 @@ See the included CLI script L<idxdb>.
 =head1 FUNCTIONS
 
 
+=head2 graph_daily_trading_summary
+
+Usage:
+
+ graph_daily_trading_summary(%args) -> [status, msg, payload, meta]
+
+Show graph from daily trading summary of some stock.
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<date_end> => I<date> (default: 1611187200)
+
+=item * B<date_start> => I<date> (default: 1608595200)
+
+=item * B<fields> => I<array[str]> (default: ["AccumForeignNetBuy"])
+
+=item * B<stock>* => I<idx::listed_stock_code>
+
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (payload) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
+
 =head2 graph_ownership
 
 Usage:
@@ -672,9 +919,9 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<date_end> => I<date>
+=item * B<date_end> => I<date> (default: 1611187200)
 
-=item * B<date_start> => I<date>
+=item * B<date_start> => I<date> (default: 1608595200)
 
 =item * B<field> => I<str> (default: "ForeignTotal")
 
@@ -710,9 +957,9 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<date_end> => I<date>
+=item * B<date_end> => I<date> (default: 1611187200)
 
-=item * B<date_start> => I<date>
+=item * B<date_start> => I<date> (default: 1608595200)
 
 =item * B<stock>* => I<idx::listed_stock_code>
 
@@ -742,6 +989,46 @@ Usage:
 
 Show ownership legend (e.g. ForeignIB = foreign bank).
 
+Examples:
+
+=over
+
+=item * Example #1:
+
+ legend_ownership();
+
+Result:
+
+ [
+   200,
+   "OK",
+   {
+     ForeignCP    => "Foreign corporate",
+     ForeignFD    => "Foreign foundation",
+     ForeignIB    => "Foreign bank",
+     ForeignID    => "Foreign individual",
+     ForeignIS    => "Foreign insurance",
+     ForeignMF    => "Foreign mutual fund",
+     ForeignOT    => "Foreign other",
+     ForeignPF    => "Foreign pension fund",
+     ForeignSC    => "Foreign securities",
+     ForeignTotal => "Foreign total",
+     LocalCP      => "Local corporate",
+     LocalFD      => "Local foundation",
+     LocalIB      => "Local bank",
+     LocalID      => "Local individual",
+     LocalIS      => "Local insurance",
+     LocalMF      => "Local mutual fund",
+     LocalOT      => "Local other",
+     LocalPF      => "Local pension fund",
+     LocalSC      => "Local securities",
+     LocalTotal   => "Local total",
+   },
+   {},
+ ]
+
+=back
+
 This function is not exported.
 
 No arguments.
@@ -759,6 +1046,46 @@ Return value:  (any)
 
 
 
+=head2 table_daily_trading_summary
+
+Usage:
+
+ table_daily_trading_summary(%args) -> [status, msg, payload, meta]
+
+Show daily trading summary of some stock.
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<date_end> => I<date> (default: 1611187200)
+
+=item * B<date_start> => I<date> (default: 1608595200)
+
+=item * B<fields> => I<array[str]> (default: ["Volume","Value","ForeignNetBuy"])
+
+=item * B<stock>* => I<idx::listed_stock_code>
+
+=item * B<total> => I<bool>
+
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (payload) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
+
 =head2 table_ownership
 
 Usage:
@@ -773,9 +1100,9 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<date_end> => I<date>
+=item * B<date_end> => I<date> (default: 1611187200)
 
-=item * B<date_start> => I<date>
+=item * B<date_start> => I<date> (default: 1608595200)
 
 =item * B<fields> => I<array[str]> (default: ["LocalTotal","ForeignTotal"])
 
